@@ -2,6 +2,7 @@ import time
 import win32api, win32con, win32gui, win32clipboard, win32print
 from typing import *
 import os
+import json
 
 
 class KeyCode:
@@ -128,7 +129,6 @@ class MouseEvent:
     BG_MOUSELEFT = (win32con.WM_LBUTTONDOWN, win32con.MK_LBUTTON, win32con.WM_LBUTTONUP)
     BG_MOUSERIGHT = (win32con.WM_RBUTTONDOWN, win32con.MK_RBUTTON, win32con.WM_RBUTTONUP)
     MOUSEWHEEL = win32con.MOUSEEVENTF_WHEEL
-    BG_MOUSEWHEEL = (win32con.WM_MOUSEWHEEL, win32con.MK_MBUTTON)
 
 class __MessageType:
     """
@@ -259,6 +259,16 @@ class Message(__MessageType):
         """
         win32gui.MessageBox(hwnd, message, title, message_type)
 
+    @staticmethod
+    def SetText(hwnd, text: str):
+        """
+        向部件中设置文字信息的操作，也可以使用InsertMessage
+        :param hwnd: 进行输入的窗口句柄
+        :param text: 需要输入的文字信息
+        :return:
+        """
+        win32api.SendMessage(hwnd, win32con.WM_SETTEXT, None, text)
+
 class ClipBoard:
     @staticmethod
     def SetClipboard(message: str, hwnd=None) -> None:
@@ -287,6 +297,17 @@ class ClipBoard:
 
 
 class Window:
+
+    @staticmethod
+    def WindowClick(father_hwnd, hwnd):
+        """
+        进行窗口部件点击
+        :param father_hwnd: 父窗口句柄
+        :param hwnd: 当前部件的句柄值
+        :return:
+        """
+        win32api.SendMessage(father_hwnd, win32con.WM_COMMAND, 1, hwnd)
+
     @staticmethod
     def GetSystemMetrics() -> tuple:
         """
@@ -353,27 +374,33 @@ class Window:
             raise RuntimeError("Need Parameter: classname or windowtext")
 
     @staticmethod
-    def GetWindowInfo(hwnd_list: Union[List[int], Tuple[int]], classname: bool = False, title: bool = False) -> List[dict]:
+    def GetWindowInfo(hwnd_list: Union[List[int], Tuple[int]], classname: bool = False, title: bool = False, beauty_json=False) -> Union[dict, str]:
         """
         获取窗口信息，class、title
         :param hwnd_list: 需要进行获取的句柄列表(或者元组)
         :param classname: 布尔值，是否获取class
         :param title: 布尔值，是否获取title
-        :return: 列表嵌套字典的形式返回
+        :param beauty_json: 是否转换为更符合打印的json格式
+        :return: 字典或者字符串json格式返回
         """
-        window_info = []
+        window_info = {}
         if any([classname, title]):
             for handle in hwnd_list:
                 if classname is False:
                     title = win32gui.GetWindowText(handle)
-                    window_info.append({handle: title})
+                    # window_info.append({handle: title})
+                    window_info[handle] = title
                 elif title is False:
                     classname = win32gui.GetClassName(handle)
-                    window_info.append({handle: classname})
+                    # window_info.append({handle: classname})
+                    window_info[handle] = classname
                 else:
                     title = win32gui.GetWindowText(handle)
                     classname = win32gui.GetClassName(handle)
-                    window_info.append({handle: {"title": title, "class": classname}})
+                    # window_info.append({handle: {"title": title, "class": classname}})
+                    window_info[handle] = {"title": title, "class": classname}
+            if beauty_json:
+                window_info = json.dumps(window_info, indent=2, ensure_ascii=True, sort_keys=True)
             return window_info
         else:
             raise ValueError("Need Parameter: classname or title")
@@ -422,11 +449,14 @@ class Window:
             raise ValueError("Only one type can be set")
 
     @staticmethod
-    def GetCurrentWindow() -> dict:
+    def GetCurrentWindow(beauty_json=False) -> Union[dict, str]:
         window = win32gui.GetForegroundWindow()
         classname = win32gui.GetClassName(window)
         title = win32gui.GetWindowText(window)
-        return {window: {"title": title, "class": classname}}
+        window_dict = {window: {"title": title, "class": classname}}
+        if beauty_json:
+            window_dict = json.dumps(window_dict, indent=2, ensure_ascii=True, sort_keys=True)
+        return window_dict
 
     @staticmethod
     def GetWindowRect(hwnd) -> dict:
@@ -439,7 +469,7 @@ class Window:
         return {"left": left, "top": top, "right": right, "bottom": bottom}
 
     @staticmethod
-    def MarkChildWindow(hwnd, details_info=False):
+    def MarkChildWindow(hwnd, details_info=False) -> list:
         """
         标记窗口子句柄位置，需要安装numpy、opencv和pillow来使用
         保存标记和原始图片到当前文件同级的Win32Handle文件夹下
@@ -503,6 +533,36 @@ class Window:
         cv2.imwrite(f'{str_pic_path}/mark-{hwnd}.png', father_window_color)     # 保存标注的图片
         return child_window
 
+    @staticmethod
+    def FuzzySearchWindow(classname: str = None, title: str = None, father_hwnd=None, beauty_json=False) -> Union[dict, str]:
+        """
+        模糊搜索窗口句柄
+        :param classname: 窗口的class属性
+        :param title: 窗口的title
+        :param father_hwnd: 父级句柄，填写则仅搜索此句柄下的子句柄
+        :param beauty_json: 是否转换为更好打印的json格式
+        :return: 字典或者字符串json形式返回窗口句柄和title、class值
+        """
+        search_res = {}
+        windows = Window.GetAllWindows(father_hwnd)
+        for window in windows:
+            win_title = win32gui.GetWindowText(window)
+            win_class = win32gui.GetClassName(window)
+            if all([classname, title]):
+                if (title in win_title) and (classname in win_class):
+                    search_res[window] = {"title": win_title, "class": win_class}
+            elif any([classname, title]):
+                if (classname is not None) and (classname in win_class):
+                    search_res[window] = {"title": win_title, "class": win_class}
+                elif (title is not None) and (title in win_title):
+                    search_res[window] = {"title": win_title, "class": win_class}
+            else:
+                raise ValueError("Need class or window title")
+        if beauty_json:
+            search_res = json.dumps(search_res, indent=2, ensure_ascii=True, sort_keys=True)
+        return search_res
+
+
 class Custom:
     """
     这里主要进行一些win32的功能封装，把某些常用的场景封装使用
@@ -514,7 +574,7 @@ class Custom:
         comboxex = Window.FindWindow('ComboBoxEx32', None, window)         # 获取子句柄combobox的句柄值
         combox = Window.FindWindow('ComboBox', None, comboxex)
         input_ = Window.FindWindow('Edit', None, combox)                 # 获取combobox下的输入框句柄值
-        win32api.SendMessage(input_, win32con.WM_SETTEXT, None, file)    # 进行文件输入
+        Message.SetText(input_, file)                                    # 进行文件输入
         win32api.SendMessage(window, win32con.WM_COMMAND, 1, button)     # 点击打开按钮进行上传
 
 
